@@ -10,4 +10,4 @@ class QueryException(BaseException):
         self.data = data
 
     def __str__(self):
-        return (%s : %s) %(data['returnCode'], data['returnMsg'])
+        return "%s : %s" % (self.data['returnCode'], self.data['returnMsg'])
