@@ -38,9 +38,9 @@ class DataApi:
             res = requests.get(url, headers=headers, timeout=self.__timeout)
         else:
             res = requests.post(url, json=post_data, headers=headers, timeout=self.__timeout)
-
+        if res.status_code != 200:
+            res.raise_for_status()
         result = json.loads(res.text)
-
         return result
 
     def get(self, url):
