@@ -6,4 +6,4 @@ from hbshare.cta_factor.factor_func import (
     factor_plot,
     factor_compute
 )
-from hbshare.cta_factor.factor_algo import carry, tsmom, tswr, tsrev, xsrev, xsmom
+from hbshare.cta_factor.factor_algo import carry, tsmom, tswr, tsrev, xsrev, xsmom, xswr
