@@ -1,5 +1,5 @@
 #!/usr/bin/python
-#coding:utf-8
+# coding:utf-8
 import pandas as pd
 import numpy as np
 
@@ -340,13 +340,15 @@ def xswr(start_date, price_field='CLOSE', lag=1, **kwargs):
 
         data_t = pd.merge(data_liq, data_t, on='UCODE', how='inner')
 
-        data_tomorrow = data[np.array(data['TDATE'] == calendar[i + 1])]
+        data_tomorrow = data[
+            np.array(data['TDATE'] == calendar[i + 1])
+        ][['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', price_field]]
 
         data_t = pd.merge(data_t[['UCODE', price_field]], data_wr_range, on='UCODE', how='inner')
 
         data_t = pd.merge(
             data_t.rename(columns={price_field: 'CLOSE'}),
-            data_tomorrow.rename(columns={'CLOSE': 'CLOSE1'}),
+            data_tomorrow.rename(columns={price_field: 'CLOSE1'}),
             on='UCODE', how='left'
         )
 
@@ -371,12 +373,13 @@ def xswr(start_date, price_field='CLOSE', lag=1, **kwargs):
 
 
 # n日净持仓
-# 净多做多
-def mr(start_date, lag=1, **kwargs):
+# 净头寸区间总和为正做多
+def mr(start_date, price_field='CLOSE', lag=1, **kwargs):
     data = kwargs['data']
     data_mr = kwargs['data_mr']
+    liq_days = kwargs['liq_days']
     data['UCODE'] = data['EXCHANGE'].apply(lambda x: str(x)) + data['PCODE'].apply(lambda x: str(x))
-    data_mr['UCODE'] = data_mr['EXCHANGE'] * 1000 + data_mr['PCODE']
+    data_mr['UCODE'] = data_mr['EXCHANGE'].apply(lambda x: str(x)) + data_mr['PCODE'].apply(lambda x: str(x))
     data_mr['SIDE'] = data_mr['SIDE'].apply(lambda x: 1 if x == 3 else (-1 if x == 4 else 0))
 
     data_mr['MR'] = data_mr['MR'] * data_mr['SIDE']
@@ -391,165 +394,183 @@ def mr(start_date, lag=1, **kwargs):
     positions = pd.DataFrame()
     for i in range(start_index, len(calendar) - 1):
         t_date = calendar[i]
-        # data_mr_0 = data_mr[
-        #     data_mr['TDATE'] == calendar[i - 1 - window_days]
-        #     ].groupby(by=['UCODE']).sum()[['MR']].reset_index()
+
         data_mr_t = data_mr[
             np.array(data_mr['TDATE'] >= calendar[i - lag - window_days])
             & np.array(data_mr['TDATE'] <= calendar[i - lag])
             ].groupby(by=['UCODE']).sum()[['MR']].reset_index()
         data_mr_range = data_mr_t
-        # data_mr_range = pd.merge(
-        #     data_mr_0.rename(columns={'MR': 'MR0'}),
-        #     data_mr_t.rename(columns={'MR': 'MR1'}),
-        #     on='UCODE', how='left'
-        # )
+
         data_mr_range['FACTORVALUE'] = data_mr_range['MR']
         data_mr_range['POS'] = data_mr_range['FACTORVALUE'] / data_mr_range['FACTORVALUE'].abs()
         data_mr_range['POS'] = data_mr_range['POS'].fillna(0)
 
-        data_t = data[
-            np.array(data['TDATE'] == t_date) & np.array(data['VOL'] > kwargs['min_volume'])
-            ][['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', 'CLOSE']]
-
-        data_tomorrow = data[np.array(data['TDATE'] == calendar[i + 1])]
-
-        data_t = pd.merge(data_t[['UCODE', 'CLOSE']], data_mr_range, on='UCODE', how='inner')
-
-        data_t = pd.merge(
-            data_t,
-            data_tomorrow.rename(columns={'CLOSE': 'CLOSE1'}),
-            on='UCODE', how='left'
-        )
-
-        data_t['RETURN'] = (data_t['CLOSE1'] / data_t['CLOSE'] - 1) #* data_t['POS']
-
-        # index_all.append(index_all[-1] * (1 + data_t['RETURN'].mean()))
-        data_t['TDATE'] = calendar[i + 1]
-        data_t['FACTOR'] = index_name
-        if sum(data_t['POS'] != 0) > 0:
-            data_t['WEIGHT'] = 1 / sum(data_t['POS'] != 0)
-        else:
-            data_t['WEIGHT'] = 0
-        positions = pd.concat([positions, data_t])
-        print(index_name + ', ' + t_date.strftime('%Y-%m-%d') + ', ' + str(sum(data_t['POS'] != 0)))
-    if len(positions) == 0:
-        return
-    positions['WEIGHT'] = positions['WEIGHT'] * 100
-    positions['RETURN'] = positions['RETURN'] * 100
-    # positions['FACTORVALUE'] = positions['FACTORVALUE'] * 100
-    return positions[
-        ['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', 'CLOSE', 'CLOSE1', 'POS', 'RETURN', 'FACTORVALUE', 'FACTOR', 'WEIGHT']
-    ].reset_index(drop=True)
-
-
-# n日净持仓变化
-# 净增做多
-def tsmr(start_date, lag=1, **kwargs):
-    data = kwargs['data']
-    data_mr = kwargs['data_mr']
-    data['UCODE'] = data['EXCHANGE'].apply(lambda x: str(x)) + data['PCODE'].apply(lambda x: str(x))
-    data_mr['UCODE'] = data_mr['EXCHANGE'] * 1000 + data_mr['PCODE']
-    data_mr['SIDE'] = data_mr['SIDE'].apply(lambda x: 1 if x == 3 else (-1 if x == 4 else 0))
-
-    data_mr['MR'] = data_mr['MR'] * data_mr['SIDE']
-
-    calendar = data['TDATE'].drop_duplicates().tolist()
-
-    window_days = kwargs['window_days']
-    index_name = 'tsmr_d' + str(window_days)
-
-    start_index = calendar.index(np.array(calendar)[np.array(calendar) >= start_date][0])
-
-    positions = pd.DataFrame()
-    for i in range(start_index, len(calendar) - 1):
-        t_date = calendar[i]
-        data_mr_0 = data_mr[
-            data_mr['TDATE'] == calendar[i - lag - window_days]
-        ].groupby(by=['UCODE']).sum()[['MR']].reset_index()
-        data_mr_t = data_mr[
-            data_mr['TDATE'] == calendar[i - lag]
-        ].groupby(by=['UCODE']).sum()[['MR']].reset_index()
-
-        data_mr_range = pd.merge(
-            data_mr_0.rename(columns={'MR': 'MR0'}),
-            data_mr_t.rename(columns={'MR': 'MR1'}),
-            on='UCODE', how='left'
-        )
-        data_mr_range['FACTORVALUE'] = data_mr_range['MR1'] - data_mr_range['MR0']
-        data_mr_range['POS'] = data_mr_range['FACTORVALUE'] / data_mr_range['FACTORVALUE'].abs()
-        data_mr_range['POS'] = data_mr_range['POS'].fillna(0)
-
-        data_t = data[
-            np.array(data['TDATE'] == t_date) & np.array(data['VOL'] > kwargs['min_volume'])
-            ][['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', 'CLOSE']]
-
-        data_tomorrow = data[np.array(data['TDATE'] == calendar[i + 1])]
-
-        data_t = pd.merge(data_t[['UCODE', 'CLOSE']], data_mr_range, on='UCODE', how='inner')
-
-        data_t = pd.merge(
-            data_t,
-            data_tomorrow.rename(columns={'CLOSE': 'CLOSE1'}),
-            on='UCODE', how='left'
-        )
-
-        data_t['RETURN'] = (data_t['CLOSE1'] / data_t['CLOSE'] - 1) #* data_t['POS']
-
-        # index_all.append(index_all[-1] * (1 + data_t['RETURN'].mean()))
-        data_t['TDATE'] = calendar[i + 1]
-        data_t['FACTOR'] = index_name
-        if sum(data_t['POS'] != 0) > 0:
-            data_t['WEIGHT'] = 1 / sum(data_t['POS'] != 0)
-        else:
-            data_t['WEIGHT'] = 0
-        positions = pd.concat([positions, data_t])
-        print(index_name + ', ' + t_date.strftime('%Y-%m-%d') + ', ' + str(sum(data_t['POS'] != 0)))
-    if len(positions) == 0:
-        return
-    positions['WEIGHT'] = positions['WEIGHT'] * 100
-    positions['RETURN'] = positions['RETURN'] * 100
-    # positions['FACTORVALUE'] = positions['FACTORVALUE'] * 100
-    return positions[
-        ['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', 'CLOSE', 'CLOSE1', 'POS', 'RETURN', 'FACTORVALUE', 'FACTOR', 'WEIGHT']
-    ].reset_index(drop=True)
-
-
-# n日净增减
-# 净增做多
-def mrchg(start_date, price_field='CLOSE', lag=1, **kwargs):
-    data = kwargs['data']
-    data_mr = kwargs['data_mr']
-    data['UCODE'] = data['EXCHANGE'].apply(lambda x: str(x)) + data['PCODE'].apply(lambda x: str(x))
-    data_mr['UCODE'] = data_mr['EXCHANGE'] * 1000 + data_mr['PCODE']
-    data_mr['SIDE'] = data_mr['SIDE'].apply(lambda x: 1 if x == 3 else (-1 if x == 4 else 0))
-
-    data_mr['MRCHG'] = data_mr['MRCHG'] * data_mr['SIDE']
-
-    calendar = data['TDATE'].drop_duplicates().tolist()
-
-    window_days = kwargs['window_days']
-    index_name = 'mrchg_d' + str(window_days)
-
-    start_index = calendar.index(np.array(calendar)[np.array(calendar) >= start_date][0])
-
-    positions = pd.DataFrame()
-    for i in range(start_index, len(calendar) - 1):
-        t_date = calendar[i]
-        data_mr_t = data_mr[
-            np.array(data_mr['TDATE'] >= calendar[i - lag - window_days])
-            & np.array(data_mr['TDATE'] <= calendar[i - lag])
-            ].groupby(by=['UCODE']).sum()[['MRCHG']].reset_index()
-        data_mr_range = data_mr_t
-
-        data_mr_range['FACTORVALUE'] = data_mr_range['MRCHG']
-        data_mr_range['POS'] = data_mr_range['FACTORVALUE'] / data_mr_range['FACTORVALUE'].abs()
-        data_mr_range['POS'] = data_mr_range['POS'].fillna(0)
-
         data_t = data[
             np.array(data['TDATE'] == t_date) & np.array(data['VOL'] > kwargs['min_volume'])
             ][['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', price_field]]
 
+        data_liq = data[
+            data['TDATE'] == calendar[i - liq_days]
+            ][['UCODE']]
+
+        data_t = pd.merge(data_liq, data_t, on='UCODE', how='inner')
+
+        data_tomorrow = data[
+            np.array(data['TDATE'] == calendar[i + 1])
+        ][['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', price_field]]
+
+        data_t = pd.merge(data_t[['UCODE', price_field]], data_mr_range, on='UCODE', how='inner')
+
+        data_t = pd.merge(
+            data_t.rename(columns={price_field: 'CLOSE'}),
+            data_tomorrow.rename(columns={price_field: 'CLOSE1'}),
+            on='UCODE', how='left'
+        )
+
+        data_t['RETURN'] = (data_t['CLOSE1'] / data_t['CLOSE'] - 1) #* data_t['POS']
+
+        # index_all.append(index_all[-1] * (1 + data_t['RETURN'].mean()))
+        data_t['TDATE'] = calendar[i + 1]
+        data_t['FACTOR'] = index_name
+        if sum(data_t['POS'] != 0) > 0:
+            data_t['WEIGHT'] = 1 / sum(data_t['POS'] != 0)
+        else:
+            data_t['WEIGHT'] = 0
+        positions = pd.concat([positions, data_t])
+        print(index_name + ', ' + t_date.strftime('%Y-%m-%d') + ', ' + str(sum(data_t['POS'] != 0)))
+    if len(positions) == 0:
+        return
+    positions['WEIGHT'] = positions['WEIGHT'] * 100
+    positions['RETURN'] = positions['RETURN'] * 100
+    # positions['FACTORVALUE'] = positions['FACTORVALUE'] * 100
+    return positions[
+        ['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', 'CLOSE', 'CLOSE1', 'POS', 'RETURN', 'FACTORVALUE', 'FACTOR', 'WEIGHT']
+    ].reset_index(drop=True)
+
+
+# n日净持仓变化
+# 净头寸时序增加做多
+def tsmr(start_date, price_field='CLOSE', lag=1, **kwargs):
+    data = kwargs['data']
+    data_mr = kwargs['data_mr']
+    liq_days = kwargs['liq_days']
+    data['UCODE'] = data['EXCHANGE'].apply(lambda x: str(x)) + data['PCODE'].apply(lambda x: str(x))
+    data_mr['UCODE'] = data_mr['EXCHANGE'].apply(lambda x: str(x)) + data_mr['PCODE'].apply(lambda x: str(x))
+    data_mr['SIDE'] = data_mr['SIDE'].apply(lambda x: 1 if x == 3 else (-1 if x == 4 else 0))
+
+    data_mr['MR'] = data_mr['MR'] * data_mr['SIDE']
+
+    calendar = data['TDATE'].drop_duplicates().tolist()
+
+    window_days = kwargs['window_days']
+    index_name = 'tsmr_d' + str(window_days)
+
+    start_index = calendar.index(np.array(calendar)[np.array(calendar) >= start_date][0])
+
+    positions = pd.DataFrame()
+    for i in range(start_index, len(calendar) - 1):
+        t_date = calendar[i]
+        data_mr_0 = data_mr[
+            data_mr['TDATE'] == calendar[i - lag - window_days]
+        ].groupby(by=['UCODE']).sum()[['MR']].reset_index()
+        data_mr_t = data_mr[
+            data_mr['TDATE'] == calendar[i - lag]
+        ].groupby(by=['UCODE']).sum()[['MR']].reset_index()
+
+        data_mr_range = pd.merge(
+            data_mr_0.rename(columns={'MR': 'MR0'}),
+            data_mr_t.rename(columns={'MR': 'MR1'}),
+            on='UCODE', how='left'
+        )
+        data_mr_range['FACTORVALUE'] = data_mr_range['MR1'] - data_mr_range['MR0']
+        data_mr_range['POS'] = data_mr_range['FACTORVALUE'] / data_mr_range['FACTORVALUE'].abs()
+        data_mr_range['POS'] = data_mr_range['POS'].fillna(0)
+
+        data_t = data[
+            np.array(data['TDATE'] == t_date) & np.array(data['VOL'] > kwargs['min_volume'])
+            ][['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', price_field]]
+
+        data_liq = data[
+            data['TDATE'] == calendar[i - liq_days]
+            ][['UCODE']]
+
+        data_t = pd.merge(data_liq, data_t, on='UCODE', how='inner')
+
+        data_tomorrow = data[
+            np.array(data['TDATE'] == calendar[i + 1])
+        ][['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', price_field]]
+
+        data_t = pd.merge(data_t[['UCODE', price_field]], data_mr_range, on='UCODE', how='inner')
+
+        data_t = pd.merge(
+            data_t.rename(columns={price_field: 'CLOSE'}),
+            data_tomorrow.rename(columns={price_field: 'CLOSE1'}),
+            on='UCODE', how='left'
+        )
+
+        data_t['RETURN'] = (data_t['CLOSE1'] / data_t['CLOSE'] - 1) #* data_t['POS']
+
+        # index_all.append(index_all[-1] * (1 + data_t['RETURN'].mean()))
+        data_t['TDATE'] = calendar[i + 1]
+        data_t['FACTOR'] = index_name
+        if sum(data_t['POS'] != 0) > 0:
+            data_t['WEIGHT'] = 1 / sum(data_t['POS'] != 0)
+        else:
+            data_t['WEIGHT'] = 0
+        positions = pd.concat([positions, data_t])
+        print(index_name + ', ' + t_date.strftime('%Y-%m-%d') + ', ' + str(sum(data_t['POS'] != 0)))
+    if len(positions) == 0:
+        return
+    positions['WEIGHT'] = positions['WEIGHT'] * 100
+    positions['RETURN'] = positions['RETURN'] * 100
+    # positions['FACTORVALUE'] = positions['FACTORVALUE'] * 100
+    return positions[
+        ['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', 'CLOSE', 'CLOSE1', 'POS', 'RETURN', 'FACTORVALUE', 'FACTOR', 'WEIGHT']
+    ].reset_index(drop=True)
+
+
+# n日净增减
+# 净变化为正做多
+def mrchg(start_date, price_field='CLOSE', lag=1, **kwargs):
+    data = kwargs['data']
+    data_mr = kwargs['data_mr']
+    liq_days = kwargs['liq_days']
+    data['UCODE'] = data['EXCHANGE'].apply(lambda x: str(x)) + data['PCODE'].apply(lambda x: str(x))
+    data_mr['UCODE'] = data_mr['EXCHANGE'].apply(lambda x: str(x)) + data_mr['PCODE'].apply(lambda x: str(x))
+    data_mr['SIDE'] = data_mr['SIDE'].apply(lambda x: 1 if x == 3 else (-1 if x == 4 else 0))
+
+    data_mr['MRCHG'] = data_mr['MRCHG'] * data_mr['SIDE']
+
+    calendar = data['TDATE'].drop_duplicates().tolist()
+
+    window_days = kwargs['window_days']
+    index_name = 'mrchg_d' + str(window_days)
+
+    start_index = calendar.index(np.array(calendar)[np.array(calendar) >= start_date][0])
+
+    positions = pd.DataFrame()
+    for i in range(start_index, len(calendar) - 1):
+        t_date = calendar[i]
+        data_mr_t = data_mr[
+            np.array(data_mr['TDATE'] >= calendar[i - lag - window_days])
+            & np.array(data_mr['TDATE'] <= calendar[i - lag])
+            ].groupby(by=['UCODE']).sum()[['MRCHG']].reset_index()
+        data_mr_range = data_mr_t
+
+        data_mr_range['FACTORVALUE'] = data_mr_range['MRCHG']
+        data_mr_range['POS'] = data_mr_range['FACTORVALUE'] / data_mr_range['FACTORVALUE'].abs()
+        data_mr_range['POS'] = data_mr_range['POS'].fillna(0)
+
+        data_t = data[
+            np.array(data['TDATE'] == t_date) & np.array(data['VOL'] > kwargs['min_volume'])
+            ][['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', price_field]]
+
+        data_liq = data[
+            data['TDATE'] == calendar[i - liq_days]
+            ][['UCODE']]
+
+        data_t = pd.merge(data_liq, data_t, on='UCODE', how='inner')
+
         data_tomorrow = data[
             np.array(data['TDATE'] == calendar[i + 1])
         ][['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', price_field]]
