@@ -61,7 +61,7 @@ def load_calendar_extra(start_date=None, end_date=None, freq='', db_path=''):
 # 净值第一次变化的时间点为数据起点（此条取消）
 def load_funds_data(
         fund_list, first_date=None, end_date=None, freq='w', fillna=True, all_aligned=False,
-        db_path='', cal_db_path='', print_info=False
+        db_path='', cal_db_path='', print_info=False, drop_1=False,
 ):
     if first_date is None:
         first_date = datetime.now().date()
@@ -146,10 +146,11 @@ def load_funds_data(
         fund_data = pd.merge(cal_spe, fund_data, on='t_date', how='left')
 
         # 剔除产品运行前期净值长期横盘情况
-        # for d in range(len(fund_data)):
-        #     if fund_data.iloc[d:d + 1][fund_name][d] == 1 and fund_data.iloc[d + 1:d + 2][fund_name][d + 1] != 1:
-        #         fund_data = fund_data.iloc[d:].reset_index(drop=True)
-        #         break
+        if drop_1:
+            for d in range(len(fund_data)):
+                if fund_data.iloc[d:d + 1][fund_name][d] == 1 and fund_data.iloc[d + 1:d + 2][fund_name][d + 1] != 1:
+                    fund_data = fund_data.iloc[d:].reset_index(drop=True)
+                    break
         try:
             i_dates.append(fund_data['t_date'][fund_data[fund_name] > 0].tolist()[0])
         except IndexError:
