@@ -30,7 +30,7 @@ class StyleAttributionOutputs:
             "r_square_series": self.r_square_series,
             "exposure_series": self.exposure_series,
             "return_attr_series": self.return_attr_series,
-            "style_all_bar": self.style_allo_bar,
+            "style_allo_bar": self.style_allo_bar,
             "style_allo_line": self.style_allo_line
         }
 
@@ -122,7 +122,9 @@ class StyleAttribution:
             benchmark_nav_series = self.benchmark_series.reindex(date_list).interpolate().reindex(
                 trading_day_list).dropna().sort_index()
             benchmark_nav_series = benchmark_nav_series / benchmark_nav_series.iloc[0]
-            benchmark_return_series = benchmark_nav_series.pct_change().dropna()
+            # benchmark_return_series = benchmark_nav_series.pct_change().dropna()
+            benchmark_return_series = benchmark_nav_series.pct_change().dropna().reindex(
+                portfolio_return_series.index).fillna(0.)
 
             assert(portfolio_return_series.shape[0] == factor_return_df.shape[0])
             assert(portfolio_return_series.shape[0] == benchmark_return_series.shape[0])
@@ -354,5 +356,6 @@ if __name__ == '__main__':
     # 私募样例
     # res = StyleAttribution(fund_id='110011', fund_type='mutual', start_date='20190220', end_date='20210120',
     #                        factor_type='sector', benchmark_id='000300', nav_frequency='day').get_all()
-    res = StyleAttribution(fund_id='P11697', fund_type='private', start_date='20151220', end_date='20210621',
-                           factor_type='style_allo', benchmark_id='000985', nav_frequency='week').get_all()
+    res = StyleAttribution(fund_id='110011', fund_type='mutual', start_date='20201220', end_date='20210702',
+                           factor_type='style', benchmark_id='000985', nav_frequency='day').get_all(processed=True)
+    print(res['exposure_series'])
